//****************************************************************************//
// Environment Mapping Demo
//****************************************************************************//

//****************************************************************************//
// Includes
//****************************************************************************//

#include <iostream>
#include <fstream>
#include <string>
#include <windows.h>
#include <glut.h>
#include <glext.h>
using namespace std;

//****************************************************************************//
// Global variables
//****************************************************************************//

int g_width = 800, g_height = 600;
int g_angles[2] = {45.0f , 30.0f};
GLuint g_cubeTextureId;

//****************************************************************************//
// GLUT callback functions
//****************************************************************************//

void displayFunc()
{
  // check for valid OpenGL state
  if(glGetError() != GL_NO_ERROR) exit(-1);

  // set the viewport
  glViewport(0, 0, g_width, g_height);

  // clear all the buffers
  glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

  // set the projection transformation
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(45.0f, (GLdouble)g_width / (GLdouble)g_height, 0.1, 100.0);

  // set the model-view transformation
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  // specify the light source
  GLfloat ambientLight[]    = { 0.2f, 0.2f, 0.2f, 1.0f };
  GLfloat diffuseLight[]    = { 0.5f, 0.5f, 0.8f, 1.0f };
  GLfloat specularLight[]   = { 1.0f, 1.0f, 1.0f, 1.0f };
  GLfloat specularReflect[] = { 1.0f, 1.0f, 1.0f, 1.0f };
  GLfloat lightPos[]        = { -50.f, 50.0f, 75.0f, 1.0f };
	glLightfv(GL_LIGHT0, GL_AMBIENT, ambientLight);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuseLight);
	glLightfv(GL_LIGHT0, GL_SPECULAR, specularLight);
  glLightfv(GL_LIGHT0, GL_POSITION, lightPos);

  // enable lighting and light source
  glEnable(GL_LIGHT0);
	glEnable(GL_LIGHTING);

	// enable the color tracking/material settings
	glEnable(GL_COLOR_MATERIAL);
	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
	glMaterialfv(GL_FRONT, GL_SPECULAR, specularReflect);
	glMateriali(GL_FRONT, GL_SHININESS, 128);
  glShadeModel(GL_SMOOTH);

  // set the camera position/angles
  glTranslatef(0.0f, 0.0f, -5.0f);
  glRotatef(g_angles[1], 1, 0, 0);
  glRotatef(g_angles[0], 0, 0, 1);

  // enable the environment mapping setup
  glEnable(GL_TEXTURE_CUBE_MAP_ARB); 
  glBindTexture(GL_TEXTURE_CUBE_MAP_ARB, g_cubeTextureId);
  glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_ARB);
  glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_ARB);
  glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_ARB);
  glEnable(GL_TEXTURE_GEN_S); 
  glEnable(GL_TEXTURE_GEN_T); 
  glEnable(GL_TEXTURE_GEN_R);

  // render our teapot
  glEnable(GL_DEPTH_TEST);
  glColor3f(1.0f, 1.0f, 1.0f);
  glutSolidTeapot(1.0f);
  glDisable(GL_DEPTH_TEST);

  // disable the environment mapping setup
  glDisable(GL_TEXTURE_GEN_S); 
  glDisable(GL_TEXTURE_GEN_T); 
  glDisable(GL_TEXTURE_GEN_R);
  glDisable(GL_TEXTURE_CUBE_MAP_ARB); 

  // disable the color tracking
	glDisable(GL_COLOR_MATERIAL);

  // disable lighting and light source
  glEnable(GL_LIGHT0);
	glEnable(GL_LIGHTING);

  // swap the front- and back-buffer
  glutSwapBuffers();
}

void idleFunc()
{
  // update the screen
  glutPostRedisplay();
}

void motionFunc(int x, int y)
{
  // set the camera angles
  g_angles[0] = (int)(x * 1.0f);
  g_angles[1] = (int)(y * 1.0f);
}

void reshapeFunc(int width, int height)
{
  // set the new window size
  g_width = width;
  g_height = height;
}

//****************************************************************************//
// Load a texture map from a given file
//****************************************************************************//

bool loadTexture(const string& strFilename, GLuint textureId, GLenum target, GLenum target2)
{
  // open the texture file
  ifstream file;
  file.open(strFilename.c_str(), ios::in | ios::binary);
  if(!file)
  {
    cerr << "Texture file '" << strFilename << "' not found." << endl;
    return false;
  }

  // allocate a temporary buffer to hold the texture data
  unsigned char *pBuffer;
  pBuffer = new unsigned char[512 * 512 * 3];
  if(pBuffer == 0)
  {
    cerr << "Memory allocation for texture file '" << strFilename << "' failed." << endl;
    return false;
  }

  // load the texture
  file.read((char *)pBuffer, 512 * 512 * 3);
  file.close();

  // generate the texture
  glBindTexture(target, textureId);
  glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
  glTexParameteri(target, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
  glTexParameteri(target, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
  glTexImage2D(target2, 0, GL_RGB, 512, 512, 0, GL_RGB, GL_UNSIGNED_BYTE, pBuffer);

  // free the temporary buffer
  delete [] pBuffer;

  return true;
}

//****************************************************************************//
// Main entry point of the application
//****************************************************************************//

int main(int argc, char *argv[])
{
  // initialize the GLUT system
  glutInit(&argc, argv);

  // set all GLUT modes
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH | GLUT_STENCIL);
  glutInitWindowSize(g_width, g_height);
  glutCreateWindow("- Environment Mapping Demo -");

  // register all GLUT callback functions
  glutIdleFunc(idleFunc);
  glutMotionFunc(motionFunc);
  glutPassiveMotionFunc(motionFunc);
  glutReshapeFunc(reshapeFunc);
  glutDisplayFunc(displayFunc);

  // load the environment texture
  glGenTextures(1, &g_cubeTextureId);
  if(!loadTexture("posx.raw", g_cubeTextureId, GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB)) return -1;
  if(!loadTexture("negx.raw", g_cubeTextureId, GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB)) return -1;
  if(!loadTexture("posy.raw", g_cubeTextureId, GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB)) return -1;
  if(!loadTexture("negy.raw", g_cubeTextureId, GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB)) return -1;
  if(!loadTexture("posz.raw", g_cubeTextureId, GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB)) return -1;
  if(!loadTexture("negz.raw", g_cubeTextureId, GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB)) return -1;

  // run the GLUT message loop
  glutMainLoop();

  return 0;
}

//****************************************************************************//
